@@ -114,7 +114,7 @@ class DestriperResult:
 
 
 @njit
-def _accumulate_map_and_info(tod, pix, psi, weights, info):
+def _accumulate_map_and_info(tod, pix, psi, weights, info, additional_component: bool):
     # Fill the upper triangle of the information matrix and use the lower
     # triangle for the RHS of the map-making equation
     assert tod.shape == pix.shape == psi.shape
@@ -127,12 +127,15 @@ def _accumulate_map_and_info(tod, pix, psi, weights, info):
             cos = np.cos(2 * a) / np.sqrt(weights[idet])
             sin = np.sin(2 * a) / np.sqrt(weights[idet])
             info_pix = info[p]
-            info_pix[0, 0] += one * one
-            info_pix[0, 1] += one * cos
-            info_pix[0, 2] += one * sin
-            info_pix[1, 1] += cos * cos
-            info_pix[1, 2] += sin * cos
-            info_pix[2, 2] += sin * sin
+
+            if not additional_component:
+                info_pix[0, 0] += one * one
+                info_pix[0, 1] += one * cos
+                info_pix[0, 2] += one * sin
+                info_pix[1, 1] += cos * cos
+                info_pix[1, 2] += sin * cos
+                info_pix[2, 2] += sin * sin
+
             info_pix[1, 0] += d * one * one
             info_pix[2, 0] += d * cos * one
             info_pix[2, 1] += d * sin * one
@@ -154,6 +157,7 @@ def make_bin_map(
     pointings: Union[np.ndarray, List[np.ndarray], None] = None,
     do_covariance: bool = False,
     output_map_in_galactic: bool = True,
+    components: List[str] = ["tod"],
 ):
     """Bin Map-maker
 
@@ -163,10 +167,11 @@ def make_bin_map(
         obss (list of :class:`Observations`): observations to be mapped. They
             are required to have the following attributes as arrays
 
-            * `tod`: the time-ordered data to be mapped
             * `pointings`: the pointing information (in radians) for each tod
                sample
             * `psi`: the polarization angle (in radians) for each tod sample
+            * any attribute listed in `components` (by default, `tod`) and
+              containing the TOD(s) to be binned together
 
             If the observations are distributed over some communicator(s), they
             must share the same group processes.
@@ -233,9 +238,10 @@ def make_bin_map(
         except AttributeError:
             weights = np.ones(cur_obs.n_detectors)
 
-        ndets = cur_obs.tod.shape[0]
-        pixidx_all = np.empty_like(cur_obs.tod, dtype=int)
-        polang_all = np.empty_like(cur_obs.tod)
+        first_component = getattr(cur_obs, components[0])
+        ndets = first_component.shape[0]
+        pixidx_all = np.empty_like(first_component, dtype=int)
+        polang_all = np.empty_like(first_component)
 
         for idet in range(ndets):
             if output_map_in_galactic:
@@ -249,7 +255,21 @@ def make_bin_map(
             pixidx_all[idet] = hpx.ang2pix(curr_pointings_det)
             polang_all[idet] = curr_pol_angle_det
 
-        _accumulate_map_and_info(cur_obs.tod, pixidx_all, polang_all, weights, info)
+        for idx, cur_component_name in enumerate(components):
+            cur_component = getattr(cur_obs, cur_component_name)
+            assert (
+                cur_component.shape == first_component.shape
+            ), 'The two TODs "{}" and "{}" do not have a matching shape'.format(
+                components[0], cur_component_name
+            )
+            _accumulate_map_and_info(
+                cur_component,
+                pixidx_all,
+                polang_all,
+                weights,
+                info,
+                additional_component=idx > 0,
+            )
 
     if all([obs.comm is None for obs in obs_list]) or not mpi.MPI_ENABLED:
         # Serial call
