@@ -15,6 +15,8 @@ from .coordinates import rotate_coordinates_e2g, CoordinateSystem
 
 from .healpix import npix_to_nside
 
+import logging
+
 
 @njit
 def compute_signal_for_one_sample(T, Q, U, co, si):
@@ -91,6 +93,7 @@ def scan_map_in_observations(
     maps: Dict[str, np.ndarray],
     pointings: Union[np.ndarray, List[np.ndarray], None] = None,
     input_map_in_galactic: bool = True,
+    component: str = "tod",
 ):
     """Scan a map filling time-ordered data
 
@@ -99,6 +102,18 @@ def scan_map_in_observations(
     bed a :class:`.Observation` instance and a NumPy matrix, or a list
     of observations and a list of NumPy matrices; in the latter case, they must have
     the same number of elements.
+
+    By default, the signal is added to ``Observation.tod``. If you want to add it to
+    some other field of the :class:`.Observation` class, use `component`::
+
+        for cur_obs in sim.observations:
+            # Allocate a new TOD for the sky signal alone
+            cur_obs.sky_tod = np.zeros_like(cur_obs.tod)
+
+        # Ask `add_noise_to_observations` to store the noise
+        # in `obs.sky_tod`
+        scan_map_in_observations(sim.observations, …, component="sky_tod")
+
     """
 
     if pointings is None:
@@ -144,6 +159,16 @@ def scan_map_in_observations(
                     "The dictionary maps does not contain all the relevant"
                     + "keys, please check the list of detectors and channels"
                 )
+            if "Coordinates" in maps.keys():
+                dict_input_map_in_galactic = (
+                    maps["Coordinates"] is CoordinateSystem.Galactic
+                )
+                if dict_input_map_in_galactic != input_map_in_galactic:
+                    logging.warning(
+                        "input_map_in_galactic variable in scan_map_in_observations"
+                        + " overwritten!"
+                    )
+                input_map_in_galactic = dict_input_map_in_galactic
         else:
             assert isinstance(maps, np.ndarray), (
                 "maps must either a dictionary contaning keys for all the"
@@ -152,7 +177,7 @@ def scan_map_in_observations(
             input_names = None
 
         scan_map(
-            tod=cur_obs.tod,
+            tod=getattr(cur_obs, component),
             pointings=cur_ptg,
             pol_angle=cur_psi,
             maps=maps,
