@@ -2,26 +2,35 @@
 
 import codecs
 from collections import namedtuple
-from dataclasses import asdict
+from dataclasses import asdict, dataclass
 from datetime import datetime
+from deprecation import deprecated
 import logging as log
 import os
 import subprocess
-from typing import List, Tuple, Union, Dict, Any
+from typing import List, Tuple, Union, Dict, Any, Optional
 from pathlib import Path
 from shutil import copyfile, copytree, SameFileError
 
-from .detectors import DetectorInfo
+import litebird_sim
+from . import HWP
+from .detectors import DetectorInfo, InstrumentInfo
 from .distribute import distribute_evenly, distribute_optimally
-from .healpix import write_healpix_map_to_file
+from .healpix import write_healpix_map_to_file, npix_to_nside
 from .imo.imo import Imo
 from .mpi import MPI_COMM_WORLD
 from .observations import Observation
+from .pointings import get_pointings
 from .version import (
     __version__ as litebird_sim_version,
     __author__ as litebird_sim_author,
 )
 
+from .dipole import DipoleType, add_dipole_to_observations
+from .scan_map import scan_map_in_observations
+from .spacecraft import SpacecraftOrbit, spacecraft_pos_and_vel
+from .noise import add_noise_to_observations
+
 import astropy.time
 import astropy.units
 import markdown
@@ -34,6 +43,8 @@ from markdown_katex import KatexExtension
 from .scanning import ScanningStrategy, SpinningScanningStrategy
 
 
+DEFAULT_BASE_IMO_URL = "https://litebirdimo.ssdc.asi.it"
+
 OutputFileRecord = namedtuple("OutputFileRecord", ["path", "description"])
 
 
@@ -97,6 +108,105 @@ def get_template_file_path(filename: Union[str, Path]) -> Path:
     return Path(__file__).parent / ".." / "templates" / filename
 
 
+@dataclass
+class MpiObservationDescr:
+    """
+    This class is used within :class:`.MpiProcessDescr`. It describes the
+    kind and size of the data held by a :class:`.Observation` object.
+
+    Its fields are:
+
+    - `det_names` (list of ``str``): names of the detectors handled by
+      this observation
+    - `tod_names` (list of ``str``): names of the fields containing the TODs
+      (e.g., ``tod``, ``cmb_tod``, ``dipole_tod``, …)
+    - `tod_shape` (tuples of ``int``): shape of each TOD held by the observation.
+      This is *not* a list, because all the TODs are assumed to have the same shape
+    - `tod_dtype` (list of ``str``): string representing the NumPy data type of each
+      TODs, in the same order as in the field `tod_name`
+    - `start_time` (either a ``float`` or a ``astropy.time.Time``): start date
+      of the observation
+    - `duration_s` (``float``): duration of the TOD in seconds
+    - `num_of_samples` (``int``): number of samples held by this TOD
+    - `num_of_detectors` (``int``): number of detectors held by this TOD. It's
+      the length of the field `det_names` (see above)
+    """
+
+    det_names: List[str]
+    tod_names: List[str]
+    tod_shape: Optional[Tuple[int, int]]
+    tod_dtype: List[str]
+    start_time: Union[float, astropy.time.Time]
+    duration_s: float
+    num_of_samples: int
+    num_of_detectors: int
+
+
+@dataclass
+class MpiProcessDescr:
+    """
+    Description of the kind of data held by a MPI process
+
+    This class is used within :class:`MpiDistributionDescr`. Its fields are:
+
+    - `mpi_rank`: rank of the MPI process described by this instance
+    - `observations`: list of :class:`.MpiObservationDescr` objects, each
+      describing one observation managed by the MPI process with rank
+      `mpi_rank`.
+
+    """
+
+    mpi_rank: int
+    observations: List[MpiObservationDescr]
+
+
+@dataclass
+class MpiDistributionDescr:
+    """A class that describes how observations are distributed among MPI processes
+
+    The fields defined in this dataclass are the following:
+
+    - `num_of_observations` (int): overall number of observations in *all* the
+      MPI processes
+    - `detectors` (list of :class:`.DetectorInfo` objects): list of *all* the
+      detectors used in the observations
+    - `mpi_processes`: list of :class:`.MpiProcessDescr` instances, describing
+      the kind of data that each MPI process is currently holding
+
+    Use :meth:`.Simulation.describe_mpi_distribution` to get an instance of this
+    object."""
+
+    num_of_observations: int
+    detectors: List[DetectorInfo]
+    mpi_processes: List[MpiProcessDescr]
+
+    def __repr__(self):
+        result = ""
+        for cur_mpi_proc in self.mpi_processes:
+            result += f"# MPI rank #{cur_mpi_proc.mpi_rank + 1}\n\n"
+            for cur_obs_idx, cur_obs in enumerate(cur_mpi_proc.observations):
+                result += """## Observation #{obs_idx}
+- Start time: {start_time}
+- Duration: {duration_s} s
+- {num_of_detectors} detector(s) ({det_names})
+- TOD(s): {tod_names}
+- TOD shape: {tod_shape}
+- Type of the TODs: {tod_dtype}
+
+""".format(
+                    obs_idx=cur_obs_idx,
+                    start_time=cur_obs.start_time,
+                    duration_s=cur_obs.duration_s,
+                    num_of_detectors=len(cur_obs.det_names),
+                    det_names=",".join(cur_obs.det_names),
+                    tod_names=", ".join(cur_obs.tod_names),
+                    tod_shape="×".join([str(x) for x in cur_obs.tod_shape]),
+                    tod_dtype=", ".join(cur_obs.tod_dtype),
+                )
+
+        return result
+
+
 class Simulation:
     """A container object for running simulations
 
@@ -129,7 +239,7 @@ class Simulation:
             print(f"{curpath}: {curdescr}")
 
     When pointing information is needed, you can call the method
-    :meth:`.Simulation.generate_spin2ecl_quaternions`, which
+    :meth:`.Simulation.set_scanning_strategy`, which
     initializes the members `pointing_freq_hz` and
     `spin2ecliptic_quats`; these members are used by functions like
     :func:`.get_pointings`.
@@ -191,6 +301,10 @@ class Simulation:
         self.start_time = start_time
         self.duration_s = duration_s
 
+        self.detectors = []  # type: List[DetectorInfo]
+        self.instrument = None  # type: Optional[InstrumentInfo]
+        self.hwp = None  # type: Optional[HWP]
+
         self.spin2ecliptic_quats = None
 
         self.description = description
@@ -484,7 +598,9 @@ class Simulation:
                 OutputFileRecord(path=curpath, description="Figure")
             )
 
-    def _fill_dictionary_with_imo_information(self, dictionary: Dict[str, Any]):
+    def _fill_dictionary_with_imo_information(
+        self, dictionary: Dict[str, Any], base_imo_url: str
+    ):
         # Fill the variable "dictionary" with information about the
         # objects retrieved from the IMO. This is used when producing
         # the final report for a simulation
@@ -523,6 +639,7 @@ class Simulation:
         dictionary["quantities"] = quantities
         dictionary["data_files"] = data_files
         dictionary["warnings"] = warnings
+        dictionary["base_imo_url"] = base_imo_url
 
     def _fill_dictionary_with_code_status(self, dictionary, include_git_diff):
         # Fill the variable "dictionary" with information about the
@@ -574,7 +691,7 @@ class Simulation:
                 f"unable to save information about latest git commit in the report: {e}"
             )
 
-    def flush(self, include_git_diff=True):
+    def flush(self, include_git_diff=True, base_imo_url: str = DEFAULT_BASE_IMO_URL):
         """Terminate a simulation.
 
         This function must be called when a simulation is complete. It
@@ -586,7 +703,9 @@ class Simulation:
         """
 
         dictionary = {"datetime": datetime.now().strftime("%Y-%m-%d %H:%M:%S")}
-        self._fill_dictionary_with_imo_information(dictionary)
+        self._fill_dictionary_with_imo_information(
+            dictionary, base_imo_url=base_imo_url
+        )
         self._fill_dictionary_with_code_status(dictionary, include_git_diff)
 
         template_file_path = get_template_file_path("report_appendix.md")
@@ -691,11 +810,18 @@ class Simulation:
             self.duration_s, (float, int)
         ), "you must set duration_s when creating the Simulation object"
 
+        if not detectors:
+            detectors = self.detectors
+
+        # if a single detector is passed, make it a list
+        if isinstance(detectors, DetectorInfo):
+            detectors = [detectors]
+
         observations = []
 
         duration_s = self.duration_s  # Cache the value to a local variable
         sampfreq_hz = detectors[0].sampling_rate_hz
-        detectors = [asdict(d) for d in detectors]
+        self.detectors = detectors
         num_of_samples = int(sampfreq_hz * duration_s)
         samples_per_obs = distribute_evenly(num_of_samples, num_of_obs_per_detector)
 
@@ -704,7 +830,7 @@ class Simulation:
         for cur_obs_idx in range(num_of_obs_per_detector):
             nsamples = samples_per_obs[cur_obs_idx].num_of_elements
             cur_obs = Observation(
-                detectors=detectors,
+                detectors=[asdict(d) for d in detectors],
                 start_time_global=cur_time,
                 sampling_rate_hz=sampfreq_hz,
                 n_samples_global=nsamples,
@@ -745,12 +871,96 @@ class Simulation:
             span.start_idx : (span.start_idx + span.num_of_elements)
         ]
 
-    def generate_spin2ecl_quaternions(
+    def describe_mpi_distribution(
+        self, tod_names: List[str] = ["tod"]
+    ) -> Optional[MpiDistributionDescr]:
+        """Return a :class:`.MpiDistributionDescr` object describing observations
+
+        This method returns a :class:`.MpiDistributionDescr` that describes the data
+        stored in each MPI process running concurrently. It is a great debugging tool
+        when you are using MPI, and it can be used for tasks where you have to carefully
+        orchestrate they way different MPI processes run together.
+
+        The method registers the amount of memory required by each TOD; by default, only
+        the ``tod`` field of each observation is considered, but you can pass more than
+        one of them through the parameter ``tod_names`` (a list of strings).
+
+        If this method is called before :meth:`.Simulation.create_observations`, it will
+        return ``None``.
+
+        This method should be called by *all* the MPI processes. It can be executed in a
+        serial environment (i.e., without MPI) and will still return meaningful values.
+
+        The typical usage for this method is to call it once you have called
+        :meth:`.Simulation.create_observations` to check that the TODs have been
+        laid in memory in the way you expect::
+
+            sim.create_observations(…)
+            distr = sim.describe_mpi_distribution()
+            if litebird_sim.MPI_COMM_WORLD.rank == 0:
+                print(distr)
+
+        """
+
+        if not self.observations:
+            return None
+
+        observation_descr = []  # type: List[MpiObservationDescr]
+        for obs in self.observations:
+            cur_det_names = list(obs.name)
+
+            shapes = [tuple(getattr(obs, name).shape) for name in tod_names]
+            # Check that all the TODs have the same shape
+            if shapes:
+                for i in range(1, len(shapes)):
+                    assert shapes[0] == shapes[i], (
+                        f"TOD {tod_names[0]} and {tod_names[i]} have different shapes: "
+                        + f"{shapes[0]} vs {shapes[i]}"
+                    )
+
+            observation_descr.append(
+                MpiObservationDescr(
+                    det_names=cur_det_names,
+                    tod_names=tod_names,
+                    tod_shape=shapes[0] if shapes else None,
+                    tod_dtype=[getattr(obs, name).dtype.name for name in tod_names],
+                    start_time=obs.start_time,
+                    duration_s=obs.n_samples / obs.sampling_rate_hz,
+                    num_of_samples=obs.n_samples,
+                    num_of_detectors=obs.n_detectors,
+                )
+            )
+
+        num_of_observations = len(self.observations)
+
+        if self.mpi_comm and litebird_sim.MPI_ENABLED:
+            observation_descr_all = MPI_COMM_WORLD.allgather(observation_descr)
+            num_of_observations_all = MPI_COMM_WORLD.allgather(num_of_observations)
+        else:
+            observation_descr_all = [observation_descr]
+            num_of_observations_all = [num_of_observations]
+
+        mpi_processes = []  # type: List[MpiProcessDescr]
+        for i in range(MPI_COMM_WORLD.size):
+            mpi_processes.append(
+                MpiProcessDescr(
+                    mpi_rank=i,
+                    observations=observation_descr_all[i],
+                )
+            )
+
+        return MpiDistributionDescr(
+            num_of_observations=sum(num_of_observations_all),
+            detectors=self.detectors,
+            mpi_processes=mpi_processes,
+        )
+
+    def set_scanning_strategy(
         self,
         scanning_strategy: Union[None, ScanningStrategy] = None,
         imo_url: Union[None, str] = None,
         delta_time_s: float = 60.0,
-        append_to_report=True,
+        append_to_report: bool = True,
     ):
         """Simulate the motion of the spacecraft in free space
 
@@ -774,17 +984,18 @@ class Simulation:
 
         The parameter `delta_time_s` specifies how often should
         quaternions be computed; see
-        :meth:`.ScanningStrategy.generate_spin2ecl_quaternions` for
+        :meth:`.ScanningStrategy.set_scanning_strategy` for
         more information.
 
         If the parameter `append_to_report` is set to ``True`` (the
         default), some information about the pointings will be included
-        in the report saved by the :class:`.Simulation` object.
+        in the report saved by the :class:`.Simulation` object. This will
+        be done only if the process has rank #0.
 
         """
         assert not (scanning_strategy and imo_url), (
             "you must either specify scanning_strategy or imo_url (but not"
-            "the two together) when calling Simulation.generate_spin2ecl_quaternions"
+            "the two together) when calling Simulation.set_scanning_strategy"
         )
 
         if not scanning_strategy:
@@ -803,12 +1014,262 @@ class Simulation:
         )
         quat_memory_size_bytes = self.spin2ecliptic_quats.nbytes()
 
-        template_file_path = get_template_file_path("report_generate_pointings.md")
-        with template_file_path.open("rt") as inpf:
-            markdown_template = "".join(inpf.readlines())
-        self.append_to_report(
-            markdown_template,
-            num_of_obs=len(self.observations),
+        num_of_obs = len(self.observations)
+        if append_to_report and litebird_sim.MPI_ENABLED:
+            num_of_obs = MPI_COMM_WORLD.allreduce(num_of_obs)
+
+        if append_to_report and MPI_COMM_WORLD.rank == 0:
+            template_file_path = get_template_file_path("report_quaternions.md")
+            with template_file_path.open("rt") as inpf:
+                markdown_template = "".join(inpf.readlines())
+            self.append_to_report(
+                markdown_template,
+                num_of_obs=num_of_obs,
+                num_of_mpi_processes=MPI_COMM_WORLD.size,
+                delta_time_s=delta_time_s,
+                quat_memory_size_bytes=quat_memory_size_bytes,
+            )
+
+    @deprecated(
+        deprecated_in="0.9",
+        current_version=litebird_sim_version,
+        details="Use set_scanning_strategy",
+    )
+    def generate_spin2ecl_quaternions(
+        self,
+        scanning_strategy: Union[None, ScanningStrategy] = None,
+        imo_url: Union[None, str] = None,
+        delta_time_s: float = 60.0,
+        append_to_report=True,
+    ):
+        self.set_scanning_strategy(
+            scanning_strategy=scanning_strategy,
+            imo_url=imo_url,
             delta_time_s=delta_time_s,
-            quat_memory_size_bytes=quat_memory_size_bytes,
+            append_to_report=append_to_report,
         )
+
+    def set_instrument(self, instrument: InstrumentInfo):
+        """Set the instrument to be used in the simulation.
+
+        This function sets the ``self.instrument`` field to the instance
+        of the class :class:`.InstrumentInfo` that has been passed as
+        argument. The purpose of the instrument is to provide the reference
+        frame for the direction of each detector.
+
+        Note that you should not simulate more than one instrument in the same
+        simulation. This is enforced by the fact that if you call `set_instrument`
+        twice, the second call will overwrite the instrument that was formerly
+        set.
+        """
+        self.instrument = instrument
+
+    def set_hwp(self, hwp: HWP):
+        """Set the HWP to be used in the simulation
+
+        The argument must be a class derived from :class:`.HWP`, for instance
+        :class:`.IdealHWP`.
+        """
+        self.hwp = hwp
+
+    def compute_pointings(
+        self,
+        append_to_report: bool = True,
+        dtype_quaternion=np.float64,
+        dtype_pointing=np.float32,
+    ):
+        """Trigger the computation of pointings.
+
+        This method must be called after having set the scanning strategy, the
+        instrument, and the list of detectors to simulate through calls to
+        :meth:`.set_instrument` and :meth:`.add_detector`. It combines the
+        quaternions of the spacecraft, of the instrument, and of the detectors
+        and sets the fields ``pointings``, ``psi``, and ``pointing_coords`` in
+        each observation owned by the simulation.
+        """
+        assert self.detectors, (
+            "You must call Simulation.create_observations() "
+            "before calling Simulation.compute_pointings"
+        )
+        assert self.instrument
+        assert self.spin2ecliptic_quats
+
+        memory_occupation = 0
+        num_of_obs = 0
+        for cur_obs in self.observations:
+            get_pointings(
+                cur_obs,
+                self.spin2ecliptic_quats,
+                detector_quats=cur_obs.quat,
+                bore2spin_quat=self.instrument.bore2spin_quat,
+                hwp=self.hwp,
+                dtype_quaternion=dtype_quaternion,
+                dtype_pointing=dtype_pointing,
+                store_pointings_in_obs=True,
+            )
+            memory_occupation += cur_obs.pointings.nbytes + cur_obs.psi.nbytes
+            num_of_obs += 1
+
+        if append_to_report and litebird_sim.MPI_ENABLED:
+            memory_occupation = MPI_COMM_WORLD.allreduce(memory_occupation)
+            num_of_obs = MPI_COMM_WORLD.allreduce(num_of_obs)
+
+        if append_to_report and MPI_COMM_WORLD.rank == 0:
+            template_file_path = get_template_file_path("report_pointings.md")
+            with template_file_path.open("rt") as inpf:
+                markdown_template = "".join(inpf.readlines())
+            self.append_to_report(
+                markdown_template,
+                num_of_obs=num_of_obs,
+                hwp_description=str(self.hwp) if self.hwp else "No HWP",
+                num_of_mpi_processes=MPI_COMM_WORLD.size,
+                memory_occupation=memory_occupation,
+            )
+
+    def compute_pos_and_vel(
+        self,
+        delta_time_s=86400.0,
+        solar_velocity_km_s: float = 369.8160,
+        solar_velocity_gal_lat_rad: float = 0.842_173_724,
+        solar_velocity_gal_lon_rad: float = 4.608_035_744_4,
+    ):
+        """Computes the position and the velocity of the spacescraft for computing
+        the dipole.
+        It wraps the :class:`.SpacecraftOrbit` and calls :meth:`.SpacecraftOrbit`.
+        The parameters that can be modified are the sampling of position and velocity
+        and the direction and amplitude of the solar dipole.
+        Default values for solar dipole from Planck 2018 Solar dipole (see arxiv:
+        1807.06207)
+        """
+
+        orbit = SpacecraftOrbit(
+            self.start_time,
+            solar_velocity_km_s=solar_velocity_km_s,
+            solar_velocity_gal_lat_rad=solar_velocity_gal_lat_rad,
+            solar_velocity_gal_lon_rad=solar_velocity_gal_lon_rad,
+        )
+
+        self.pos_and_vel = spacecraft_pos_and_vel(
+            orbit=orbit, obs=self.observations, delta_time_s=delta_time_s
+        )
+
+    def fill_tods(
+        self,
+        maps: Dict[str, np.ndarray],
+        append_to_report: bool = True,
+    ):
+        """Fills the TODs, scanning a map.
+
+        This method must be called after having set the scanning strategy, the
+        instrument, the list of detectors to simulate through calls to
+        :meth:`.set_instrument` and :meth:`.add_detector`, and the methond
+        compute_pointings. maps is assumed to be produced by :class:`.Mbs`
+        """
+
+        scan_map_in_observations(
+            self.observations,
+            maps=maps,
+        )
+
+        if append_to_report and MPI_COMM_WORLD.rank == 0:
+            template_file_path = get_template_file_path("report_scan_map.md")
+            with template_file_path.open("rt") as inpf:
+                markdown_template = "".join(inpf.readlines())
+            if type(maps) is dict:
+                if "Mbs_parameters" in maps.keys():
+
+                    if maps["Mbs_parameters"].make_fg:
+                        fg_model = maps["Mbs_parameters"].fg_models
+                    else:
+                        fg_model = "N/A"
+
+                    self.append_to_report(
+                        markdown_template,
+                        nside=maps["Mbs_parameters"].nside,
+                        has_cmb=maps["Mbs_parameters"].make_cmb,
+                        has_fg=maps["Mbs_parameters"].make_fg,
+                        fg_model=fg_model,
+                    )
+            else:
+                nside = npix_to_nside(len(maps[0]))
+                self.append_to_report(
+                    markdown_template,
+                    nside=nside,
+                    has_cmb="N/A",
+                    has_fg="N/A",
+                    fg_model="N/A",
+                )
+
+    def add_dipole(
+        self,
+        t_cmb_k: float = 2.72548,  # Fixsen 2009 http://arxiv.org/abs/0911.1955
+        dipole_type: DipoleType = DipoleType.TOTAL_FROM_LIN_T,
+        append_to_report: bool = True,
+    ):
+        """Fills the tod with dipole.
+
+        This method must be called after having set the scanning strategy, the
+        instrument, the list of detectors to simulate through calls to
+        :meth:`.set_instrument` and :meth:`.add_detector`, and the pointing
+        through :meth:`.compute_pointings`.
+        """
+
+        if not hasattr(self, "pos_and_vel"):
+            self.compute_pos_and_vel()
+
+        add_dipole_to_observations(
+            obs=self.observations,
+            pos_and_vel=self.pos_and_vel,
+            t_cmb_k=t_cmb_k,
+            dipole_type=dipole_type,
+        )
+
+        if append_to_report and MPI_COMM_WORLD.rank == 0:
+            template_file_path = get_template_file_path("report_dipole.md")
+
+            dip_lat_deg = np.rad2deg(self.pos_and_vel.orbit.solar_velocity_gal_lat_rad)
+            dip_lon_deg = np.rad2deg(self.pos_and_vel.orbit.solar_velocity_gal_lon_rad)
+            dip_velocity = self.pos_and_vel.orbit.solar_velocity_km_s
+
+            with template_file_path.open("rt") as inpf:
+                markdown_template = "".join(inpf.readlines())
+            self.append_to_report(
+                markdown_template,
+                t_cmb_k=t_cmb_k,
+                dipole_type=dipole_type,
+                dip_lat_deg=dip_lat_deg,
+                dip_lon_deg=dip_lon_deg,
+                dip_velocity=dip_velocity,
+            )
+
+    def add_noise(
+        self,
+        noise_type: str = "one_over_f",
+        random: Union[np.random.Generator, None] = None,
+        append_to_report: bool = True,
+    ):
+
+        """Adds noise to tods.
+
+        This method must be called after having set the instrument,
+        the list of detectors to simulate through calls to
+        :meth:`.set_instrument` and :meth:`.add_detector`.
+        """
+
+        if random is None:
+            random = self.random
+
+        add_noise_to_observations(
+            obs=self.observations,
+            noise_type=noise_type,
+            random=random,
+        )
+
+        if append_to_report and MPI_COMM_WORLD.rank == 0:
+            template_file_path = get_template_file_path("report_noise.md")
+            with template_file_path.open("rt") as inpf:
+                markdown_template = "".join(inpf.readlines())
+            self.append_to_report(
+                markdown_template,
+                noise_type="white + 1/f " if noise_type == "one_over_f" else "white",
+            )
